/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.emailer

import scala.xml._
import java.net.URL
import scala.io._
import scala.collection.Map
import scala.collection.mutable.HashMap
import javax.mail.internet._
import javax.mail.Message
import javax.mail.Transport

object Main {
  def main(args: Array[String]) {
    val api = new URL("http://quoridor.xelfi.cz/api/");

    val id = login(api, "emailer", "otravuj")
    
    val ret = allPlayers(
      api,
      3600 * 1000 * 24
    )

    val props = java.lang.System.getProperties
    props.put("mail.smtp.host", "192.168.1.1");

    val session = javax.mail.Session.getDefaultInstance(props, null)


    for (val address <- emails(api, id, ret.keySet)) {
        Console.println("Sending message to " + address)

        val message = new MimeMessage(session)
        message.setFrom(new InternetAddress("quoridor@xelfi.cz"))
        message.addRecipient(Message.RecipientType.TO, new InternetAddress(address))
        message.setSubject("Play Quoridor!")
        message.setText("Visit http://quoridor.xelfi.cz")

        Transport.send(message)
    }
  }

  def login(url : URL, name : String, password : String) : String = {
    val u = new URL(url, "login?name=" + name + "&password=" + password);
    val conn = u.openConnection.asInstanceOf[java.net.HttpURLConnection]
    conn.setRequestMethod("PUT")
    conn.setDoInput(true)
    conn.connect
    val reply = Source.fromInputStream(conn.getInputStream)
    return reply.mkString("")
  }

  def allPlayers(url : URL, okDelay : Long): Map[String,List[Node]] = {
    val u = new URL(url, "games");
    val conn = u.openConnection
    conn.setRequestProperty("Accept", "text/xml")

    val ret = new HashMap[String,List[Node]]

    val before = System.currentTimeMillis - okDelay
    val games = XML.load(conn.getInputStream)
    for (val g : Node <- games \\ "gameId") {
      val modified = (g \\ "@modified").text.toLong
      if (modified < before) {
        val status = (g \\ "@status").text;
        val name =
          if (status == "blackMove") {
            (g \\ "@black").text
          } else if (status == "whiteMove") {
            (g \\ "@white").text
          } else {
            null
          }
        if (name != null) {
          val prev = ret.getOrElse(name, Nil)
          val next = g :: prev
          ret.put(name, next);
        }
      }
    }
    return ret;
  }

  def emails(url : URL, loginID : String, ids : Collection[String]): List[String] = {
    var ret : List[String] = Nil;
    for (val p <- ids) {
      val u = new URL(url, "users/" + p + "?loginID=" + loginID)
      val conn = u.openConnection
      conn.setRequestProperty("Accept", "text/xml")
      val xml = XML.load(conn.getInputStream)
      val res = (xml \\ "property").filter({
          n => n.attribute("name") match {
            case Some(attr) => attr.text == "email"
            case _ => false
          }
      })
      if (!res.isEmpty) {
        ret = res(0).text :: ret;
      }
    }
    return ret
  }
}
